#-*-coding:utf-8-*-
"""
    Interface de synchronisation entre le ldap et la bdd posh
    >>> from posh.api.scribe_api import ScribePosh
    >>> sync_handler = ScribePosh()
    Premier usage : synchronisation d'un groupe
    >>> sync_handler.synchronize_group('mongroupe')

    Second usage : synchronisation d'un type de groupe (Classe par défaut)
    >>> sync_handler.synchronize('Classe')

    Troisième usage : synchronisation forcée
                (recrée le groupe même si le cache pense qu'il y est déjà)
    >>> sync_handler.synchronize_group('mongroupe', force=True)
    >>> sync_handler.synchronize(force=True)
"""
import pickle
from os.path import isfile

from scribe.eolegroup import Group
from scribe.eoleuser import User

from posh.logger.logger import getLogger
from posh.api.user_api import PoshUserApi
from posh.api.group_api import PoshGroupApi
from posh.api.usergroup_api import PoshUserGroupApi

log = getLogger("api.scribe_api")

SPEC_GROUPS = [] #['professeurs', 'eleves', 'administratifs']

GROUPIDS_FILE = "/tmp/posh_groupids.pickle"
USERGROUPIDS_FILE = "/tmp/posh_usergroupids.pickle"

def load_groupids(filename):
    """
        Charge les ids des groupes dans la base posh
        {nomdugroupe:{id:iddugroupe, parentid:iddugroupeparent(facultatif)},..}
    """
    if isfile(filename):
        return pickle.load(file(filename, 'r'))
    else:
        return {}

def save_groupids(filename, groupids_dict):
    """
        Sauvegarde les ids de groupe
    """
    pickle.dump(groupids_dict, file(filename, 'w'))
    return True

class ScribePosh(PoshUserApi, PoshGroupApi, PoshUserGroupApi):
    """
        Interface de synchronisation entre le ldap et la bdd posh
        .synchronize_usergroups(_type=typedegroupe, force=True/False)
        .synchronize_usergroup(groupname, force=True/False)
        .purge(groupname)
        .purge()
    """

    def __init__(self):
        super(ScribePosh, self).__init__()
        self.scribeuser = User()
        self.scribegroup = Group()
        self.scribeuser.ldap_admin = self.scribegroup.ldap_admin
        self.load_usergroupids()
        self.load_groupids()

# FONCTIONS PUBLIQUES (à utiliser)
    def synchronize(self):
        """
            Synchronise les classes de l'annuaire
        """
        self.synchronize_usergroups()
        self.synchronize_groups()

    def synchronize_usergroups(self, _type='Classe', force=False):
        """
            Synchronise les groupes de type _type du ldap
            en tant que usergroup
        """
        print("#          Synchronisation des groupes de type : %s \
comme groupe utilisateur          #" % _type)
        log.info("####    Synchronisation des groupes de type : %s \
comme groupe utilisateur    ####" % _type)
        self.scribegroup.ldap_admin.connect()
        for groupname in self.scribegroup._get_groups(_type=_type):
            self._synchronize_usergroup(groupname, force=force, _type=_type)
        self.scribegroup.ldap_admin.close()
        self.save_usergroupids()

    def synchronize_usergroup(self, groupname, force=False):
        """
            Synchonise le groupe groupname comme groupe de carnet
        """
        self.scribegroup.ldap_admin.connect()
        self._synchronize_usergroup(groupname, force=force)
        self.scribegroup.ldap_admin.close()
        self.save_usergroupids()

    def synchronize_groups(self, _type='Classe', force=False):
        """
            Synchronise les groupes de _type comme groupe de profile
        """
        log.info("####    Synchronisation des groupes de type : %s \
comme groupe de profil    ####" % _type)
        self.scribegroup.ldap_admin.connect()
        for groupname in self.scribegroup._get_groups(_type=_type):
            self._synchronize_group(groupname, force=force, _type=_type)
        self.scribegroup.ldap_admin.close()
        self.save_groupids()

    def synchronize_group(self, groupname, force=False):
        """
            Synchonise le groupe groupname comme groupe de profile
        """
        self.scribegroup.ldap_admin.connect()
        self._synchronize_group(groupname, force=force)
        self.scribegroup.ldap_admin.close()
        self.save_groupids()

    def synchronize_users(self):
        """
            Synchronise les utilisateurs de l'annuaire
        """
        log.info("####    Synchronisation des utilisateurs \
de l'annuaire ####")
        self.scribeuser.ldap_admin.connect()
        for user in self.scribeuser._get_users():
            self._create_user(user)
        self.scribeuser.ldap_admin.close()

    def synchronize_user(self, username):
        """
            Synchronise l'utilisateur username
        """
        self.scribeuser.ldap_admin.connect()
        self._create_user(username)
        self.scribeuser.ldap_admin.close()

    def purge(self, groupname=None):
        """
            Purge les utilisateurs de la base posh pour le groupe groupname
        """
        self.scribegroup.ldap_admin.connect()
        if groupname:
            users = self.scribegroup._get_members(groupname)
        else:
            users = self.scribeuser._get_users()
        for user in users:
            self.del_user(user)
        self.scribegroup.ldap_admin.close()

#FONCTIONS PRIVÉES (PAS DE CONNEXION LDAP)
    def _synchronize_usergroup(self, groupname, force=False, _type=None):
        """
            Synchonise le groupe groupname (n'initialise pas
            la connexion ldap préférer synchronize_usergroup)
            le groupe sera nommé typedugroupe-nomdugroupe
        """
        print(" + Synchronisation du groupe %s + " % (groupname,))
        log.info(" + Synchronisation du groupe %s + " % (groupname,))
        if not _type:
            _type = self.scribegroup.c_get_group_type(groupname)
        groupid = self._create_usergroup("%s-%s" % (_type, groupname),
                                         force=force)
        self._populate_usergroup(groupname, groupid)
        return groupname, groupid

    def _synchronize_group(self, groupname, force=False, _type=None):
        """
            Synchonise le groupe groupname comme groupe de profile
        """
        if not _type:
            _type = self.scribegroup.c_get_group_type(groupname)
        groupid = self._create_group("%s-%s" % (_type, groupname),
                                     force=force)
        self._populate_group(groupname, groupid)
        return groupname, groupid

    def _create_usergroup(self, groupname, force=False):
        """
            Crée le groupe groupname comme groupe de carnet
        """
        if force or not self.usergroupids.has_key(groupname):
            log.info("  + Création du groupe : \"%s\"" % (
                                                           groupname,))
            # crée le groupe dans la base et enregistre l'id dans le dico
            usergroupid = self.add_usergroup(groupname, 'utilisateur_lambda')
            self.usergroupids[groupname] = {'id':usergroupid}
            return usergroupid
        else:
            log.info("  + Le groupe \"%s\" existe déjà" % (
                                                           groupname,))
            return self.usergroupids[groupname]['id']

    def _create_group(self, groupname, force=False):
        """
            Crée le groupe groupname comme groupe de profile
        """
        if force or not self.groupids.has_key(groupname):
            log.info("  + Création du groupe : \"%s\"" % (
                                                            groupname,))
            # crée le groupe dans la base et enregistre l'id dans le dic
            groupid = self.add_group(groupname)
            self.groupids[groupname] = {'id':groupid}
            return groupid
        else:
            log.info("  + Le groupe \"%s\" existe déjà" % (
                                                          groupname,))
            return self.groupids[groupname]['id']

    def _populate_usergroup(self, groupname, groupid):
        """
            Peuple le groupe groupname
        """
        for user in self.scribegroup._get_members(groupname):
            userid = self._create_user(user)[0]
            log.info("   + Inscription de l'utilisateur : %s" % (user,))
            self._subscribe_usergroup(groupid, userid)
        if self.scribegroup._is_group('profs-%s' % groupname):
            for user in self.scribegroup._get_members('profs-%s' % groupname):
                userid = self._create_user(user)[0]
                log.info("   + Inscription de l'enseignant : \"%s\"" % (user,))
                self._subscribe_usergroup(groupid, userid)

    def _populate_group(self, groupname, groupid):
        """
            Peuple le groupe groupname
        """
        for user in self.scribegroup._get_members(groupname):
            username = self._create_user(user)[1]
            log.info("   + Inscription de l'utilisateur : %s" % (user,))
            self.subscribe_profilgroup(username, groupid)

    def _create_user(self, username):
        """
            Crée l'utilisateur username
        """
        try:
            longname = self.scribeuser._get_attr(username, 'displayName')[0]
        except IndexError:
            longname = username
        log.info("  + Création de l'utilisateur : %s" % (username,))
        return self.add_user(username, longname), username

    def save_usergroupids(self):
        """
            Sauve les ids des usergroups synchronisés
        """
        save_groupids(USERGROUPIDS_FILE, self.usergroupids)

    def save_groupids(self):
        """
            Sauve les ids des usergroups synchronisés
        """
        save_groupids(GROUPIDS_FILE, self.groupids)

    def load_usergroupids(self):
        """
            charge les ides des usergroups déjà synchronisé
        """
        self.usergroupids = load_groupids(USERGROUPIDS_FILE)

    def load_groupids(self):
        """
            Charge les ids de groupes déjà synchronisé
        """
        self.groupids = load_groupids(GROUPIDS_FILE)
